/***********************************************************************
**
** Xilinx, Inc. 2004          www.xilinx.com
**
** XAPP693 - Configuration and Revision Manager
**
***********************************************************************
**
** Filenames and Descriptions: 
**
**	config_mon.v -  top level connectivity file for xapp693 design
**
** Date - version : 7/19/2004 - v1.0
**
** Author:  Xilinx, Inc.
**
** Contact: e-mail  hotline@xilinx.com phone   + 1 800 255 7778
**
** Disclaimer: 
** LIMITED WARRANTY AND DISCLAIMER. These designs are
** provided to you "as is". Xilinx and its licensors make and you
** receive no warranties or conditions, express, implied,
** statutory or otherwise, and Xilinx specifically disclaims any
** implied warranties of merchantability, non-infringement, or
** fitness for a particular purpose. Xilinx does not warrant that
** the functions contained in these designs will meet your
** requirements, or that the operation of these designs will be
** uninterrupted or error free, or that defects in the Designs
** will be corrected. Furthermore, Xilinx does not warrant or
** make any representations regarding use or the results of the
** use of the designs in terms of correctness, accuracy,
** reliability, or otherwise.
**
** LIMITATION OF LIABILITY. In no event will Xilinx or its
** licensors be liable for any loss of data, lost profits, cost
** or procurement of substitute goods or services, or for any
** special, incidental, consequential, or indirect damages
** arising from the use or operation of the designs or
** accompanying documentation, however caused and on any theory
** of liability. This limitation will apply even if Xilinx
** has been advised of the possibility of such damage. This
** limitation shall apply not-withstanding the failure of the 
** essential purpose of any limited remedies herein.
**
**  Copyright (c) 2004 Xilinx, Inc.
**  All rights reserved
**
********************************************************************************/

module config_monitor (	cclk_in, wd_reset, init, prog, reconfig_sw, reconfig_cmd, done_in, rev_sel, 
								rev_sel_in, en_ext_sel, en_ext_sel_in, en_wd_in, wr_en, wd_timer, status_reg, 
								ceo);


input		cclk_in; // Note, CCLK_in is a free-running input to this design...assume 10MHz

input		wd_reset, init, reconfig_sw, reconfig_cmd, done_in, en_ext_sel_in, en_wd_in, 
			wr_en, ceo;
input		[1:0] rev_sel_in;
output	prog, en_ext_sel;
output	[1:0] wd_timer, rev_sel;
output	[2:0] status_reg;

wire		[1:0] config_state, prog_timer;
wire		[2:0] status_reg;
wire		en_wd, reset, wd_reset_ok;

// State declarations...
parameter [1:0] 
	IDLE       = 2'b00,
	MONITOR    = 2'b01, 
	PASS       = 2'b11,
	FAIL       = 2'b10;


// Monitor State Machine..
monitor_sm u1_montitor (.cclk_in(cclk_in), .ceo(ceo), .en_wd(en_wd), .init(init), .done_in(done_in), 
								.wd_timer(wd_timer), .prog_timer(prog_timer), .config_state(config_state), 
								.reset(reset), .wd_reset(wd_reset), .wd_reset_ok(wd_reset_ok)) ;  

// Program Pin to FPGAs controller..
program u1_program ( .cclk_in(cclk_in), .prog_timer(prog_timer), .reset(reset), .config_state(config_state),
							.prog(prog)) ;  

// Status Register controller..
status u1_status (.cclk_in(cclk_in), .wd_reset(wd_reset), .wd_reset_ok(wd_reset_ok), .init(init), .done_in(done_in), 
						.en_wd(en_wd), .wd_timer(wd_timer), .status_reg(status_reg), .reset(reset), 
						.config_state(config_state), .ceo(ceo));  

// Revision controller for PROM Page select..
revision u1_revision (.cclk_in(cclk_in), .en_ext_sel(en_ext_sel), .en_ext_sel_in(en_ext_sel_in), .wr_en(wr_en), 
							 .rev_sel(rev_sel), .rev_sel_in(rev_sel_in), .config_state(config_state));  


// Watchdog timer instance..
watchdog u1_watchdog (.cclk_in(cclk_in), .wd_reset(wd_reset), .en_wd(en_wd), .en_wd_in(en_wd_in), .wr_en(wr_en), 
							 .wd_timer(wd_timer), .config_state(config_state), .wd_reset_ok(wd_reset_ok), .reset(reset)) ;  


// Reset control..
assign reset = (!reconfig_sw | !reconfig_cmd);

endmodule


