/**********************************************************************
**
** Xilinx, Inc. 2004          www.xilinx.com
**
** XAPP693 - Configuration and Revision Manager
**
***********************************************************************
**
** Filenames and Descriptions: 
**
**	monitor_sm.v -  configuration/revision manager state machine
**
** Date - version : 7/19/2004 - v1.0
**
** Author:  Xilinx, Inc.
**
** Contact: e-mail  hotline@xilinx.com phone   + 1 800 255 7778
**
** Disclaimer: 
** LIMITED WARRANTY AND DISCLAIMER. These designs are
** provided to you "as is". Xilinx and its licensors make and you
** receive no warranties or conditions, express, implied,
** statutory or otherwise, and Xilinx specifically disclaims any
** implied warranties of merchantability, non-infringement, or
** fitness for a particular purpose. Xilinx does not warrant that
** the functions contained in these designs will meet your
** requirements, or that the operation of these designs will be
** uninterrupted or error free, or that defects in the Designs
** will be corrected. Furthermore, Xilinx does not warrant or
** make any representations regarding use or the results of the
** use of the designs in terms of correctness, accuracy,
** reliability, or otherwise.
**
** LIMITATION OF LIABILITY. In no event will Xilinx or its
** licensors be liable for any loss of data, lost profits, cost
** or procurement of substitute goods or services, or for any
** special, incidental, consequential, or indirect damages
** arising from the use or operation of the designs or
** accompanying documentation, however caused and on any theory
** of liability. This limitation will apply even if Xilinx
** has been advised of the possibility of such damage. This
** limitation shall apply not-withstanding the failure of the 
** essential purpose of any limited remedies herein.
**
**  Copyright (c) 2004 Xilinx, Inc.
**  All rights reserved
**
********************************************************************************/

module monitor_sm (cclk_in, ceo, en_wd, init, done_in, wd_timer, prog_timer, 
						 config_state, reset, wd_reset, wd_reset_ok);


input		cclk_in, init, done_in, en_wd, reset, ceo, wd_reset, wd_reset_ok;
input 	[1:0] wd_timer, prog_timer;
output	[1:0] config_state;

reg		[1:0] config_state, next_state;

// State declarations...
parameter [1:0] 
	IDLE       = 2'b00,
	MONITOR    = 2'b01, 
	PASS       = 2'b11,
	FAIL       = 2'b10;


// State machine block...
always @(posedge cclk_in)
if (reset)
	config_state <= IDLE;
else
	config_state <= next_state;
	
// Next state logic...
always @(prog_timer or en_wd or ceo or config_state or init or	done_in or wd_timer or wd_reset or wd_reset_ok) begin

  case(config_state)
	IDLE      : if (init && !done_in && prog_timer == 2'b11) next_state <= MONITOR;			// stay in IDLE if reconfig occurred and INIT hasn't gone high yet..
						else if (en_wd) next_state <= PASS;
						else next_state <= IDLE;
						
	MONITOR   : if (!init && !done_in) next_state <= FAIL;											// monitor watches INIT and DONE and PROM CEO..
						else if (init && !done_in && !ceo) next_state <= FAIL;                 // sent bitstream but FPGA likely never saw PREAMBLE..PROM CEO is low
						else if (init && done_in && ceo) next_state <= PASS;
						else next_state <= MONITOR;

	PASS      : if (!en_wd) next_state <= IDLE;
						else if (wd_timer == 2'b10 && !wd_reset_ok) next_state <= FAIL;			// took too long to set wd_reset_ok bit..
					 	else if (wd_timer == 2'b11 && wd_reset_ok && wd_reset) next_state <= FAIL;  				// general check of WD_TIMER..
						else next_state <= PASS;

	FAIL      : if (init && !done_in && prog_timer == 2'b11) next_state <= MONITOR;			// pulse PROG low for preset amt of time...
						else next_state <= FAIL;

	default   : next_state <= IDLE;																			// default 
  endcase
end 

endmodule
