/***********************************************************************
**
** Xilinx, Inc. 2004          www.xilinx.com
**
** XAPP693 - Configuration and Revision Manager
**
***********************************************************************
**
** Filenames and Descriptions: 
**
**	program.v -  program control block
**
** Date - version : 7/19/2004 - v1.0
**
** Author:  Xilinx, Inc.
**
** Contact: e-mail  hotline@xilinx.com phone   + 1 800 255 7778
**
** Disclaimer: 
** LIMITED WARRANTY AND DISCLAIMER. These designs are
** provided to you "as is". Xilinx and its licensors make and you
** receive no warranties or conditions, express, implied,
** statutory or otherwise, and Xilinx specifically disclaims any
** implied warranties of merchantability, non-infringement, or
** fitness for a particular purpose. Xilinx does not warrant that
** the functions contained in these designs will meet your
** requirements, or that the operation of these designs will be
** uninterrupted or error free, or that defects in the Designs
** will be corrected. Furthermore, Xilinx does not warrant or
** make any representations regarding use or the results of the
** use of the designs in terms of correctness, accuracy,
** reliability, or otherwise.
**
** LIMITATION OF LIABILITY. In no event will Xilinx or its
** licensors be liable for any loss of data, lost profits, cost
** or procurement of substitute goods or services, or for any
** special, incidental, consequential, or indirect damages
** arising from the use or operation of the designs or
** accompanying documentation, however caused and on any theory
** of liability. This limitation will apply even if Xilinx
** has been advised of the possibility of such damage. This
** limitation shall apply not-withstanding the failure of the 
** essential purpose of any limited remedies herein.
**
**  Copyright (c) 2004 Xilinx, Inc.
**  All rights reserved
**
********************************************************************************/

module program (cclk_in, prog_timer, reset, config_state, prog);

input		cclk_in, reset;
input		[1:0] config_state;
output	[1:0] prog_timer;  
output	prog;

reg		prog, reconfigure;
reg		[1:0] prog_timer;  

// State declarations...
parameter [1:0] 
	IDLE       = 2'b00,
	MONITOR    = 2'b01, 
	PASS       = 2'b11,
	FAIL       = 2'b10;


// Control for logic low pulse (prog timer)
// ensures PROG low per datasheet spec..300nS min for V-II Pro
always @(posedge reset or posedge cclk_in) begin
if (reset) begin
	prog <= 1'b0;
	reconfigure <= 1'b1;
	prog_timer <= 2'b00;
end

else case (config_state)											// puts you in IDLE state..see monitor_sm
		IDLE      : if (reconfigure && prog_timer == 2'b11)
							prog <= 1'b1;
						else if (reconfigure && prog_timer <= 2'b11) begin
							prog_timer <= prog_timer + 1;
							prog <= 1'b0;
						end
						else if (!reconfigure)
							prog <= 1'b1;
		FAIL		: if (prog_timer == 2'b11) begin
							prog <= 1'b1;
						end
						else begin	
							prog <= 1'b0;
							prog_timer <= prog_timer + 1;
						end
		
		default	:	begin
							reconfigure <= 1'b0;
							prog_timer <= 2'b00;				// cover for states MONITOR and PASS...
							prog <= 1'b1;
						end
	endcase
end

endmodule

