/***********************************************************************
**
** Xilinx, Inc. 2004          www.xilinx.com
**
** XAPP693 - Configuration and Revision Manager
**
***********************************************************************
**
** Filenames and Descriptions: 
**
**	revision.v -  PROM revision control block
**
** Date - version : 7/19/2004 - v1.0
**
** Author:  Xilinx, Inc.
**
** Contact: e-mail  hotline@xilinx.com phone   + 1 800 255 7778
**
** Disclaimer: 
** LIMITED WARRANTY AND DISCLAIMER. These designs are
** provided to you "as is". Xilinx and its licensors make and you
** receive no warranties or conditions, express, implied,
** statutory or otherwise, and Xilinx specifically disclaims any
** implied warranties of merchantability, non-infringement, or
** fitness for a particular purpose. Xilinx does not warrant that
** the functions contained in these designs will meet your
** requirements, or that the operation of these designs will be
** uninterrupted or error free, or that defects in the Designs
** will be corrected. Furthermore, Xilinx does not warrant or
** make any representations regarding use or the results of the
** use of the designs in terms of correctness, accuracy,
** reliability, or otherwise.
**
** LIMITATION OF LIABILITY. In no event will Xilinx or its
** licensors be liable for any loss of data, lost profits, cost
** or procurement of substitute goods or services, or for any
** special, incidental, consequential, or indirect damages
** arising from the use or operation of the designs or
** accompanying documentation, however caused and on any theory
** of liability. This limitation will apply even if Xilinx
** has been advised of the possibility of such damage. This
** limitation shall apply not-withstanding the failure of the 
** essential purpose of any limited remedies herein.
**
**  Copyright (c) 2004 Xilinx, Inc.
**  All rights reserved
**
********************************************************************************/

module revision ( cclk_in, en_ext_sel, en_ext_sel_in, wr_en, rev_sel, rev_sel_in, 
						config_state);


input		cclk_in, en_ext_sel_in, wr_en;
input		[1:0] rev_sel_in, config_state;
output	en_ext_sel;
output	[1:0] rev_sel;

// Registers wake up in zero state by default, thus default PAGE at pwr up = [0,0]
// and en_ext_sel is 0, enabling external page revision control..
reg		en_ext_sel;
reg		[1:0] rev_sel;


// State declarations...
parameter [1:0] 
	IDLE       = 2'b00,
	MONITOR    = 2'b01, 
	PASS       = 2'b11,
	FAIL       = 2'b10;


// Page Revision Select Logic for Platform Flash PROM..
always @(posedge cclk_in) begin

if (!wr_en) begin
	en_ext_sel <= en_ext_sel_in;
	rev_sel <= rev_sel_in;
end

// If en_ext_sel is HIGH, rev_sel doesn't matter - per Flash PROM datasheet..internal control
else if (!en_ext_sel && config_state == FAIL) 
	rev_sel <= 2'b00;
end

endmodule