/**********************************************************************
**
** Xilinx, Inc. 2004          www.xilinx.com
**
** XAPP693 - Configuration and Revision Manager
**
***********************************************************************
**
** Filenames and Descriptions: 
**
**	status.v -  Status control block
**
** Date - version : 7/19/2004 - v1.0
**
** Author:  Xilinx, Inc.
**
** Contact: e-mail  hotline@xilinx.com phone   + 1 800 255 7778
**
** Disclaimer: 
** LIMITED WARRANTY AND DISCLAIMER. These designs are
** provided to you "as is". Xilinx and its licensors make and you
** receive no warranties or conditions, express, implied,
** statutory or otherwise, and Xilinx specifically disclaims any
** implied warranties of merchantability, non-infringement, or
** fitness for a particular purpose. Xilinx does not warrant that
** the functions contained in these designs will meet your
** requirements, or that the operation of these designs will be
** uninterrupted or error free, or that defects in the Designs
** will be corrected. Furthermore, Xilinx does not warrant or
** make any representations regarding use or the results of the
** use of the designs in terms of correctness, accuracy,
** reliability, or otherwise.
**
** LIMITATION OF LIABILITY. In no event will Xilinx or its
** licensors be liable for any loss of data, lost profits, cost
** or procurement of substitute goods or services, or for any
** special, incidental, consequential, or indirect damages
** arising from the use or operation of the designs or
** accompanying documentation, however caused and on any theory
** of liability. This limitation will apply even if Xilinx
** has been advised of the possibility of such damage. This
** limitation shall apply not-withstanding the failure of the 
** essential purpose of any limited remedies herein.
**
**  Copyright (c) 2004 Xilinx, Inc.
**  All rights reserved
**
********************************************************************************/

module status (cclk_in, wd_reset, wd_reset_ok, init, done_in, en_wd, wd_timer, status_reg, 
					reset, config_state, ceo);

input		cclk_in, wd_reset, wd_reset_ok, init, done_in, en_wd, reset, ceo;
input		[1:0] wd_timer, config_state;
output	[2:0] status_reg;

reg		[2:0] status_reg;

// State declarations...
parameter [1:0] 
	IDLE       = 2'b00,
	MONITOR    = 2'b01, 
	PASS       = 2'b11,
	FAIL       = 2'b10;

// Status Register bit codes
// 000 - all good
// 001 - previous bitstream failed Watchdog timer
// 010 - previous bitstream failed CRC test
// 100 - failed PREAMBLE test, e.g. timed out never saw sync word



// Status Register Logic...
always @(posedge cclk_in)
if (reset)
 	status_reg <= 3'b000;		// default state for status register..

else 
	case (config_state) 
		MONITOR	:	if (!init && !done_in) 
							status_reg <= (status_reg | 3'b010);  // failed CRC test... e.g. INIT went low during config..
						else if (init && !done_in && !ceo) 
							status_reg <= (status_reg | 3'b100);  // failed PREAMBLE test... e.g. timed out
																			  // Logic OR function maintains history of status..
		PASS		:	if (en_wd) begin
							if (wd_timer == 2'b10 && !wd_reset_ok)
							status_reg <= (status_reg | 3'b001);  // failed to set watch dog timer reset bit 
							else if (wd_timer == 2'b11 && wd_reset_ok && wd_reset) 
							status_reg <= (status_reg | 3'b001);  // failed watch dog timer test..
						end

	endcase
endmodule