/***********************************************************************
**
** Xilinx, Inc. 2004          www.xilinx.com
**
** XAPP693 - Configuration and Revision Manager
**
***********************************************************************
**
** Filenames and Descriptions: 
**
**	watchdog.v -  watchdog control block
**
** Date - version : 7/19/2004 - v1.0
**
** Author:  Xilinx, Inc.
**
** Contact: e-mail  hotline@xilinx.com phone   + 1 800 255 7778
**
** Disclaimer: 
** LIMITED WARRANTY AND DISCLAIMER. These designs are
** provided to you "as is". Xilinx and its licensors make and you
** receive no warranties or conditions, express, implied,
** statutory or otherwise, and Xilinx specifically disclaims any
** implied warranties of merchantability, non-infringement, or
** fitness for a particular purpose. Xilinx does not warrant that
** the functions contained in these designs will meet your
** requirements, or that the operation of these designs will be
** uninterrupted or error free, or that defects in the Designs
** will be corrected. Furthermore, Xilinx does not warrant or
** make any representations regarding use or the results of the
** use of the designs in terms of correctness, accuracy,
** reliability, or otherwise.
**
** LIMITATION OF LIABILITY. In no event will Xilinx or its
** licensors be liable for any loss of data, lost profits, cost
** or procurement of substitute goods or services, or for any
** special, incidental, consequential, or indirect damages
** arising from the use or operation of the designs or
** accompanying documentation, however caused and on any theory
** of liability. This limitation will apply even if Xilinx
** has been advised of the possibility of such damage. This
** limitation shall apply not-withstanding the failure of the 
** essential purpose of any limited remedies herein.
**
**  Copyright (c) 2004 Xilinx, Inc.
**  All rights reserved
**
********************************************************************************/

module watchdog ( cclk_in, wd_reset, en_wd, en_wd_in, wr_en, wd_timer, config_state,
						wd_reset_ok, reset);

input		cclk_in, wd_reset, en_wd_in, wr_en, reset;
input		[1:0]	config_state;
output	[1:0] wd_timer;
output	en_wd, wd_reset_ok;

// Watch Dog timer selection...OFF by default
// Registers wake up in zero state by default, thus default enable watch dog is OFF at pwr up
reg		[1:0] wd_timer;  
reg		en_wd, wd_reset_ok;

// State declarations...
parameter [1:0] 
	IDLE       = 2'b00,
	MONITOR    = 2'b01, 
	PASS       = 2'b11,
	FAIL       = 2'b10;


// Watch-dog enable control..
always @(posedge cclk_in) begin
if (!wr_en) 
	en_wd <= en_wd_in;
end

// Watch-dog "kicker" to ensure wd_reset is not just being held low..
// reconfig_sw or reconfig_cmd clears bit
always @(posedge reset or negedge wd_reset) begin
if (reset)
	wd_reset_ok <= 1'b0;
else if (en_wd)
	wd_reset_ok <= 1'b1;
end


// Watch-dog timer control..
always @(negedge wd_reset or posedge cclk_in) begin
if (!wd_reset) 
	wd_timer <= 2'b00;

else if (en_wd)	
	case (config_state) 
		PASS		:	wd_timer <= wd_timer + 1;
		default	:	wd_timer <= 2'b00;
	endcase
end

endmodule
